searchNodes=[{"doc":"Dotenvy is an Elixir port of the original dotenv Ruby gem. It is designed to help applications follow the principles of the 12-factor app and its recommendation to store configuration in the environment. Unlike other configuration helpers, Dotenvy enforces no convention for the naming of your files: .env is a common choice, you may name your configuration files whatever you wish. See the strategies for examples of various use cases.","ref":"Dotenvy.html","title":"Dotenvy","type":"behaviour"},{"doc":"","ref":"Dotenvy.html#env/3","title":"Dotenvy.env/3","type":"function"},{"doc":"Reads the given env variable and converts its value to the given type . This function attempts to read a value from a local data store of sourced values; it will fall back to System.fetch_env/1 when no locally stored variable is available. This function may raise an error because type conversion is delegated to Dotenvy.Transformer.to!/2 -- see its documentation for a list of supported types. Examples iex&gt; env! ( &quot;PORT&quot; , :integer ) 5432 iex&gt; env! ( &quot;ENABLED&quot; , :boolean ) true","ref":"Dotenvy.html#env!/2","title":"Dotenvy.env!/2","type":"function"},{"doc":"Reads an env variable and converts its output or returns a default value. Use env!/2 when possible Use of env!/2 is recommended over env!/3 because it creates a stronger contract with the environment: your app literally will not start when required env variables are missing. If the given variable is set , its value is converted to the given type . The provided default value is only used when the variable is not set; the default value is returned as-is, without conversion . This allows greater control of the output. Conversion is delegated to Dotenvy.Transformer.to!/2 , which may raise an error. See its documentation for a list of supported types. This function attempts to read a value from a local data store of sourced values; it will fall back to System.fetch_env/1 when no locally stored variable is available. Examples iex&gt; env! ( &quot;PORT&quot; , :integer , 5432 ) 5433 iex&gt; env! ( &quot;NOT_SET&quot; , :boolean , %{ not : &quot;converted&quot; } ) %{ not : &quot;converted&quot; } iex&gt; System . put_env ( &quot;HOST&quot; , &quot;&quot; ) iex&gt; env! ( &quot;HOST&quot; , :string! , &quot;localhost&quot; ) ** (RuntimeError) Error converting HOST to string!: non-empty value required","ref":"Dotenvy.html#env!/3","title":"Dotenvy.env!/3","type":"function"},{"doc":"A parser implementation should receive the contents read from a file, a map of vars (with string keys, as would come from System.get_env/0 ), and a keyword list of opts . This callback is provided to help facilitate testing. See Dotenvy.Parser for the default implementation.","ref":"Dotenvy.html#c:parse/3","title":"Dotenvy.parse/3","type":"callback"},{"doc":"Like its Bash namesake command, source/2 accumulates values from the given input(s). The accumulated values are stored via a side effect function to make them available to the env!/2 and env!/3 functions. Think of source/2 as a merging operation which can accept maps (like Map.merge/2 ) or paths to env files. Inputs are processed from left to right so that values can be overridden by each subsequent input. As with Map.merge/2 , the right-most input takes precedence. Options :parser module that implements Dotenvy.parse/3 callback. Default: Dotenvy.Parser :require_files specifies which of the given files (if any) must be present. When true , all the listed files must exist. When false , none of the listed files must exist. When some of the files are required and some are optional, provide a list specifying which files are required. If a file listed here is not included in the function's files argument, it is ignored. Default: false :side_effect an arity 1 function called after the successful parsing inputs. The default is an internal function that stores the values inside a process dictionary so the values are available to the env!/2 and env!/3 functions. This option is overridable to facilitate testing. Changing it is not recommended. Examples The simplest implementation is to parse a single file by including its path: iex&gt; Dotenvy . source ( &quot;.env&quot; ) { :ok , %{ &quot;TIMEOUT&quot; =&gt; &quot;5000&quot; , &quot;DATABASE_URL&quot; =&gt; &quot;postgres://postgres:postgres@localhost/myapp&quot; , # ...etc... } } More commonly, you will source multiple files (often based on the config_env() ) and you will defer to pre-existing system variables. The most common pattern looks like this: iex&gt; Dotenvy . source ( [ &quot; \#{ config_env ( ) } .env&quot; , &quot; \#{ config_env ( ) } .override.env&quot; , System . get_env ( ) ] ) In the above example, the prod.env , dev.env , and test.env files would be version-controlled, but the *.override.env variants would be ignored, giving developers the ability to override values without needing to modify versioned files. Give Precedence to System Envs! Don't forget to include System.get_env() as the final input to source/2 so that system environment variables take precedence over values sourced from .env files. When you run a shell command like ❯ LOG_LEVEL=debug mix run , your expectation is probably that the LOG_LEVEL variable would be set to debug , overriding whatever may have been defined in your sourced .env files. Similarly, you may export env vars in your Bash profile. System env vars are not granted precedence automatically: you must explicitly include System.get_env() as the final input to source/2 . If your env files are making use of variable substitution based on system env vars, e.g. ${PWD} (see the Dotenv File Format ), then you would need to specify System.get_env() as the first argument to source/2 . For example, if your .env references the system HOME variable: # .env CACHE_DIR = $ { HOME } / cache then your source/2 command would need to make the system env vars available to the parser by including them as one of the inputs, e.g. iex&gt; Dotenvy . source ( [ System . get_env ( ) , &quot;.env&quot; ] ) Including the System.get_env() before your files means that your files have final say over the values, potentially overriding any pre-existing system env vars. In some cases, you may wish to reference the system vars both before and after your own .env files, e.g. iex&gt; Dotenvy . source ( [ System . get_env ( ) , &quot;.env&quot; , System . get_env ( ) ] ) or you may wish to cherry-pick which variables you need to make available for variable substitution: iex&gt; Dotenvy . source ( [ %{ &quot;HOME&quot; =&gt; System . get_env ( &quot;HOME&quot; ) } , &quot;.env&quot; , System . get_env ( ) ] ) This syntax favors explicitness so there is no confusion over what might have been &quot;automagically&quot; accumulated.","ref":"Dotenvy.html#source/2","title":"Dotenvy.source/2","type":"function"},{"doc":"As source/2 , but returns a map on success or raises on error.","ref":"Dotenvy.html#source!/2","title":"Dotenvy.source!/2","type":"function"},{"doc":"An input source may be either a path to an env file or a map with string keys, e.g. &quot;envs/.env&quot; or %{&quot;FOO&quot; =&gt; &quot;bar&quot;} . This allows users to specify a list of env files interspersed with other values from other sources, e.g. System.get_env() or values fetched from a secure parameter store.","ref":"Dotenvy.html#t:input_source/0","title":"Dotenvy.input_source/0","type":"type"},{"doc":"This module handles the parsing of the contents of .env files into maps with string keys. See Dotenv File Format for details on the supported file format. This implementation uses parsing over regular expressions for most of its work.","ref":"Dotenvy.Parser.html","title":"Dotenvy.Parser","type":"module"},{"doc":"Parse the given contents , substituting and merging with the given vars .","ref":"Dotenvy.Parser.html#parse/3","title":"Dotenvy.Parser.parse/3","type":"function"},{"doc":"This module provides functionality for converting string values to specific Elixir data types. These conversions were designed to operate on system environment variables, which always store string binaries.","ref":"Dotenvy.Transformer.html","title":"Dotenvy.Transformer","type":"module"},{"doc":"Converts strings into Elixir data types with support for nil-able values. Raises on error. Each type determines how to interpret the incoming string, e.g. when the type is :integer , an empty string is considered a 0 ; when :integer? is the type , and empty string is converted to nil . Remember: Use a ? suffix when an empty string should be considered nil (a.k.a. a &quot;nullable&quot; value). Use a ! suffix when an empty string is not allowed. Use this when values are required. Types See the Dotenvy.Transformer.conversion_type/0 for a description of valid conversion types. Examples iex&gt; to! ( &quot;debug&quot; , :atom ) :debug iex&gt; to! ( &quot;&quot; , :boolean ) false iex&gt; to! ( &quot;&quot; , :boolean? ) nil iex&gt; to! ( &quot;5432&quot; , :integer ) 5432 iex&gt; to! ( &quot;foo&quot; , fn val -&gt; val &lt;&gt; &quot;bar&quot; end ) &quot;foobar&quot;","ref":"Dotenvy.Transformer.html#to!/2","title":"Dotenvy.Transformer.to!/2","type":"function"},{"doc":"The conversion type specifies the target data type to which a string will be converted. For example, :integer would indicate a transformation of &quot;12&quot; to 12 . The following types are supported: :atom - converts to an atom. An empty string will be the atom :&quot;&quot; (!). :atom? - converts to an atom. An empty string will be considered nil :atom! - converts to an atom. An empty string will raise. :boolean - &quot;false&quot;, &quot;0&quot;, or an empty string &quot;&quot; will be considered boolean false . Any other non-empty value is considered true . :boolean? - as above, except an empty string will be considered nil :boolean! - as above, except an empty string will raise. :charlist - converts string to charlist. :charlist? - converts string to charlist. Empty string will be considered nil . :charlist! - as above, but an empty string will raise. :integer - converts a string to an integer. An empty string will be considered 0 . :integer? - as above, but an empty string will be considered nil . :integer! - as above, but an empty string will raise. :float - converts a string to an float. An empty string will be considered 0 . :float? - as above, but an empty string will be considered nil . :float! - as above, but an empty string will raise. :existing_atom - converts into an existing atom. Raises error if the atom does not exist. :existing_atom? - as above, but an empty string will be considered nil . :existing_atom! - as above, but an empty string will raise. :module - converts a string into an Elixir module name. Raises on error. :module? - as above, but an empty string will be considered nil . :module! - as above, but an empty string will raise. :string - no conversion (default) :string? - empty strings will be considered nil . :string! - as above, but an empty string will raise. custom function - see below. Custom Callback function When you require more control over the transformation of your value than is possible with the types provided, you can provide an arity 1 function in place of the type.","ref":"Dotenvy.Transformer.html#t:conversion_type/0","title":"Dotenvy.Transformer.conversion_type/0","type":"type"},{"doc":"This error module can be useful when writing your own custom conversion functions because special contextual information will be included with any errors. Examples Let's say your configuration needs to supply one of a set of possible values (i.e. an enum). We can define a custom function to support this and pass it as the second argument to Dotenvy.env!/2 # runtime.exs import Config import Dotenvy size_enum = fn &quot;large&quot; -&gt; :large &quot;small&quot; -&gt; :small _ -&gt; raise Dotenvy.Error , message : &quot;allowed size_enum values are large or small&quot; end config :myapp , :some_bool , env! ( &quot;SIZE&quot; , size_enum )","ref":"Dotenvy.Error.html","title":"Dotenvy.Error","type":"exception"},{"doc":"Dotenvy is an Elixir port of the original dotenv Ruby gem, compatible with mix and releases . It is designed to help the development of applications following the principles of the 12-factor app and its recommendation to store configuration in the environment.","ref":"readme.html","title":"Dotenvy","type":"extras"},{"doc":"Add dotenvy to your list of dependencies in mix.exs : def deps do [ { :dotenvy , &quot;~&gt; 0.8.0&quot; } ] end It has no dependencies.","ref":"readme.html#installation","title":"Dotenvy - Installation","type":"extras"},{"doc":"Dotenvy is designed to help configure your application at runtime , and one of the most effective places to do that is inside config/runtime.exs (available since Elixir v1.11). The Dotenvy.source/2 function can accept a single file or a list of files. When combined with Config.config_env/0 it is easy to load up environment-specifc config, e.g. source ( [ &quot; \#{ config_env ( ) } .env&quot; , &quot; \#{ config_env ( ) } .override.env&quot; , System . get_env ( ) ] ) By default, the listed files do not need to exist -- the function only needs to know where to look. This makes it easy to commit default values while still leaving the door open to developers to override values via their own configuration files. You control if and how existing system env vars are handled: usually they should take precedence over values defined in .env files, so the System.get_env() should be included as the final input supplied to source/2 . Unlike other packages, Dotenvy has no opinions about the names or locations of your dotenv config files, you just need to pass their paths to Dotenvy.source/2 or Dotenvy.source!/2 . For a simple example, we can load a single file: # config/runtime.exs import Config import Dotenvy source! ( [ &quot;.env&quot; , System . get_env ( ) ] ) config :myapp , MyApp.Repo , database : env! ( &quot;DATABASE&quot; , :string! ) , username : env! ( &quot;USERNAME&quot; , :string ) , password : env! ( &quot;PASSWORD&quot; , :string ) , hostname : env! ( &quot;HOSTNAME&quot; , :string! ) , pool_size : env! ( &quot;POOL_SIZE&quot; , :integer ) , adapter : env! ( &quot;ADAPTER&quot; , :module , Ecto.Adapters.Postgres ) , pool : env! ( &quot;POOL&quot; , :module? ) And then define your variables in the file(s) to be sourced. Dotenvy has no opinions about what you name your files; .env is merely a convention. # .env DATABASE=myapp_dev USERNAME=myuser PASSWORD=mypassword HOSTNAME=localhost POOL_SIZE=10 POOL= When you set up your application configuration in this way, you are creating a contract with the environment: Dotenvy.env!/2 will raise if the required variables have not been set or if the values cannot be properly transformed. This is an approach that works equally well for your day-to-day development and for mix releases. Read the configuration strategies for more detailed examples of how to configure your app. Refer to the &quot;dotenv&quot; ( .env ) file format for more examples and features of the supported syntax. See the Dotenvy module documentation on its functions. Note for Mix Tasks If you have authored your own Mix tasks, you must ensure that they load the application configuration in a way that is compatible with the runtime config. A good way to do this is to include Mix.Task.run(&quot;app.config&quot;) inside the run/1 implementation, e.g. def run ( _args ) do Mix.Task . run ( &quot;app.config&quot; ) # ... end If you are dealing with third-party mix tasks that fail to properly load configuration, you may need to manually call mix app.config before running them, e.g. mix do app.config other.task Defining a task alias in mix.exs is another way to accomplish this: # mix.exs defp aliases do [ &quot;other.task&quot; : [ &quot;app.config&quot; , &quot;other.task&quot; ] ]","ref":"readme.html#usage","title":"Dotenvy - Usage","type":"extras"},{"doc":"Starting with Dotenvy v0.6.0, the precedence of system env variables over parsed .env files is not defined; the :overwrite? and :vars options are no longer supported in Dotenvy.source/2 and Dotenvy.source!/2 . Instead, the source functions now accept file paths OR maps: this makes the question of variable precedence something that must be explicitly listed. The source functions act more like Map.merge/2 , accumulating values, always giving precedence to the righthand source. Most users upgrading from v0.5.0 will wish to include System.get_env() as the final input to source/2 . # in dotenvy 0.5.0 or before: source ( [ &quot; \#{ config_env ( ) } .env&quot; , &quot; \#{ config_env ( ) } .override.env&quot; ] ) # should be changed to the following in dotenvy 0.6.0: source ( [ &quot; \#{ config_env ( ) } .env&quot; , &quot; \#{ config_env ( ) } .override.env&quot; , System . get_env ( ) ] ) If you are relying on variable interpolation in your .env files, you may also need to include System.get_env() (or an equivalent subset) before you list your .env files. This is necessary to make values available to the file parser. # in dotenvy 0.5.0 or before: source ( [ &quot; \#{ config_env ( ) } .env&quot; , &quot; \#{ config_env ( ) } .override.env&quot; ] ) # should be changed to the following in dotenvy 0.6.0: source ( [ System . get_env ( ) , &quot; \#{ config_env ( ) } .env&quot; , &quot; \#{ config_env ( ) } .override.env&quot; , System . get_env ( ) ] ) The change in syntax introduced in v0.6.0 favors a declarative list of sources over opaquely inferred inputs. This also opens the door for compatibility with other value sources, e.g. secure parameter stores. Image Attribution: &quot;dot&quot; by Stepan Voevodin from the Noun Project","ref":"readme.html#upgrading-from-v0-5-0-or-before","title":"Dotenvy - Upgrading from v0.5.0 or before","type":"extras"},{"doc":"Quick reference for using Dotenvy.","ref":"cheatsheet.html","title":"Cheatsheet","type":"extras"},{"doc":"envs/ Directory The recommended location for storing your .env files is inside a dedicated envs/ directory. Code editors recognize the .env extension. Example envs/dev.env LOG_LEVEL=debug AWS_REGION=&quot;us-east-1&quot; HTTP_CLIENT=HTTPoison Version Control envs/dev.env is tracked by Git. Use it to store sensible non-sensitive defaults. envs/dev.local.env is ignored by Git. Use it to override any variables in the tracked version. This must match the values used in the runtime configuration. .gitignore * . local . env Release Compatibility For compatibility with releases , configure your builds to copy (i.e. &quot;overlay&quot;) the contents of your envs/ directory into the root of the release. mix.exs defp releases do [ my_app : [ include_executables_for : [ :unix ] , steps : [ :assemble , :tar ] , overlays : [ &quot;envs/&quot; , &quot;priv/&quot; , &quot;config/&quot; ] ] ] end Runtime Config Your config/runtime.exs is where you source your environment variables. The last argument to Dotenvy.source/2 or Dotenvy.source!/2 takes precedence. It's common to use System.get_env() as the final argument so any existing system environment variables will take precedence over anything parsed from the .env files. config/runtime.exs import Config import Dotenvy dir = System . get_env ( &quot;RELEASE_ROOT&quot; ) || &quot;envs/&quot; # In an umbrella app: # dir = System.get_env(&quot;RELEASE_ROOT&quot;) || Path.expand(&quot;./envs/&quot;) &lt;&gt; &quot;/&quot; source! ( [ &quot; \#{ dir } \#{ config_env ( ) } .env&quot; , &quot; \#{ dir } \#{ config_env ( ) } .local.env&quot; , System . get_env ( ) ] )","ref":"cheatsheet.html#setup","title":"Cheatsheet - Setup","type":"extras"},{"doc":"Dev ENV envs/dev.env PG_USERNAME = postgres PG_PASSWORD = postgres PG_HOSTNAME = localhost PG_PORT = 5432 PG_DATABASE = m_app_dev PG_POOL_SIZE = 10 PG_POOL = DBConnection.ConnectionPool PG_SSL = true Test ENV envs/test.env PG_USERNAME = postgres PG_PASSWORD = postgres PG_HOSTNAME = localhost PG_PORT = 5432 PG_DATABASE = my_app_test PG_POOL_SIZE = 10 PG_POOL = Ecto.Adapters.SQL.Sandbox PG_SSL = false Runtime Configuration config/runtime.env import Config import Dotenvy dir = System . get_env ( &quot;RELEASE_ROOT&quot; ) || &quot;envs/&quot; source! ( [ &quot; \#{ dir } \#{ config_env ( ) } .env&quot; , &quot; \#{ dir } \#{ config_env ( ) } .local.env&quot; , System . get_env ( ) ] ) config :my_app , MyApp.PGRepo , pool : env! ( &quot;PG_POOL&quot; , :module? ) , pool_size : env! ( &quot;PG_POOL_SIZE&quot; , :integer ) , database : env! ( &quot;PG_DATABASE&quot; , :string ) , username : env! ( &quot;PG_USERNAME&quot; , :string ) , password : env! ( &quot;PG_PASSWORD&quot; , :string ) , port : env! ( &quot;PG_PORT&quot; , :integer ) , hostname : env! ( &quot;PG_HOSTNAME&quot; , :string )","ref":"cheatsheet.html#example-database-configuration","title":"Cheatsheet - Example Database Configuration","type":"extras"},{"doc":"System Environment variables are always stored as strings which may need to be transformed into native Elixir data types. Used as the 2nd argument to Dotenvy.env!/2 and Dotenvy.env!/3 Conversion Type Elixir Type On Empty String :atom atom :&quot;&quot; :atom? atom nil :atom! atom raise ⚠ :boolean boolean false :boolean? boolean nil :boolean! boolean raise ⚠ :charlist charlist '' i.e. [] :charlist? charlist nil :charlist! charlist raise ⚠ :integer integer 0 :integer? integer nil :integer! integer raise ⚠ :float float 0 :float? float nil :float! float raise ⚠ :existing_atom atom :&quot;&quot; or raise :existing_atom? atom nil :existing_atom! atom raise ⚠ :module atom :&quot;Elixir.&quot; :module? atom nil :module! atom raise ⚠ :string String &quot;&quot; :string? String nil :string! String raise ⚠ Custom functions handle their own behavior. Custom Function Example PHX_IP=&quot;0, 0, 0, 0, 0, 0, 0, 0&quot; config :feenix , FeenixWeb.Endpoint , http : [ # Enable IPv6 and bind on all interfaces. ip : env! ( &quot;PHX_IP&quot; , fn ip -&gt; ip |&gt; String . split ( &quot;,&quot; ) |&gt; Enum . map ( &amp; String . trim / 1 ) |&gt; Enum . map ( &amp; String . to_integer / 1 ) |&gt; List . to_tuple ( ) end ) ] , Your custom functions can raise a Dotenvy.Error to benefit from improved messages that include helpful context about any problems, e.g. strict_boolean! = fn &quot;true&quot; -&gt; true &quot;false&quot; -&gt; false _ -&gt; raise Dotenvy.Error , message : &quot;strict_boolean! values must be either true or false&quot; end config :myapp , :some_bool , env! ( &quot;SOME_BOOL&quot; , strict_boolean! ) This will yield an error like the following: ** (RuntimeError) Error converting variable SOME_BOOL using custom function: strict_boolean! values must be either true or false","ref":"cheatsheet.html#transformations","title":"Cheatsheet - Transformations","type":"extras"},{"doc":"Although there are other places where Dotenvy may prove useful, it was designed with the config/runtime.exs in mind: most of the following use-cases will focus on that because it offers a clean and declarative way to load up the necessary variables.","ref":"strategies.html","title":"Strategies","type":"extras"},{"doc":"Configuration providers are most often invoked in the context of releases, and although they can solve certain problems that arise in production deployments, they tend to be an awkward fit for regular day-to-day development. Dotenvy seeks to normalize how configuration is loaded across environments, so having different methods depending on how you run your app is antithetical. We do not want some code that runs only in certain environments and not in others: it can make for untested or untestable code. Secondly, configuration providers sometimes shift the task of &quot;shaping&quot; the configuration out of Elixir and into some static representation (e.g. JSON or TOML). The allure of a straight-forward static file is deceiving because there is no easy way to delineate Elixir-specific subtleties such as distinguishing between keyword lists and maps. When configuration providers &quot;solve&quot; one problem, they often create another: it can require some busywork to convert values back into Elixir variable types that your application requires. For these reasons, Dotenvy does not rely on configuration providers ; dotenv files are an easier &quot;lingua franca&quot;.","ref":"strategies.html#a-note-on-configuration-providers","title":"Strategies - A Note on Configuration Providers","type":"extras"},{"doc":"The distinctions between &quot;dev&quot; and &quot;prod&quot; become less clear when we focus on configuration: ideally, the app is the same in all environments, it is only the configuration values themselves that can be described as &quot;dev&quot; or &quot;prod&quot; -- in this example they will live inside a single .env file. Let's look at the three files that will make this work: config/config.exs # compile-time config import Config config :myapp , ecto_repos : [ MyApp.Repo ] config :myapp , MyApp.Repo , migration_timestamps : [ type : :utc_datetime , inserted_at : :created_at ] config/runtime.exs import Config import Dotenvy source ( [ &quot;.env&quot; , System . get_env ( ) ] ) if config_env ( ) == &quot;test&quot; do config :myapp , MyApp.Repo , database : &quot;myapp_test&quot; , username : &quot;test-user&quot; , password : &quot;test-password&quot; , hostname : &quot;localhost&quot; , pool_size : 10 , adapter : Ecto.Adapters.Postgres , pool : Ecto.Adapters.SQL.Sandbox else config :myapp , MyApp.Repo , database : env! ( &quot;DATABASE&quot; , :string! ) , username : env! ( &quot;USERNAME&quot; , :string ) , password : env! ( &quot;PASSWORD&quot; , :string ) , hostname : env! ( &quot;HOSTNAME&quot; , :string! ) , pool_size : env! ( &quot;POOL_SIZE&quot; , :integer ) , adapter : env ( &quot;ADAPTER&quot; , :module , Ecto.Adapters.Postgres ) , pool : env! ( &quot;POOL&quot; , :module? ) end .env (dev or prod) DATABASE = myapp_dev USERNAME = myuser PASSWORD = mypassword HOSTNAME = localhost POOL_SIZE = 10 POOL = The .env shows some values suitable local development; if the app were deployed on a production box, it would be the same shape, but its values would point to a production database. For tests, values are hard-coded inside runtime.exs . This is one admittedly heavy-handed way to ensure that your test runs don't accidentally hit the wrong database, but it does mean that there is a small block of untestable code inside the if-statement. You may notice that in this example we have done away with config/dev.exs , config/test.exs , and config/prod.exs . These should be used only when your app has a legitimate compile-time need. If you can configure something at runtime, you should configure it at runtime. These extra config files are omitted to help demonstrate how the decisions about how the app should run can often be pushed into runtime.exs . This should help avoid confusion that often arises between compile-time and runtime configuration.","ref":"strategies.html#dotenv-for-dev-and-prod","title":"Strategies - Dotenv for Dev and Prod","type":"extras"},{"doc":"It is possible to use only a config.exs and a runtime.exs file to configure many Elixir applications: let the .env tell the app how to run! Consider the following setup: config/config.exs # compile-time config import Config config :myapp , ecto_repos : [ MyApp.Repo ] config :myapp , MyApp.Repo , migration_timestamps : [ type : :utc_datetime , inserted_at : :created_at ] config/runtime.exs import Config import Dotenvy source ( [ &quot;.env&quot; , &quot;.env. \\# {config_env()}&quot; , System . get_env ( ) ] ) config :myapp , MyApp.Repo , database : env! ( &quot;DATABASE&quot; , :string! ) , username : env! ( &quot;USERNAME&quot; , :string ) , password : env! ( &quot;PASSWORD&quot; , :string ) , hostname : env! ( &quot;HOSTNAME&quot; , :string! ) , pool_size : env! ( &quot;POOL_SIZE&quot; , :integer ) , adapter : env ( &quot;ADAPTER&quot; , :module , Ecto.Adapters.Postgres ) , pool : env! ( &quot;POOL&quot; , :module? ) .env (dev or prod) DATABASE = myapp_dev USERNAME = myuser PASSWORD = mypassword HOSTNAME = localhost POOL_SIZE = 10 POOL = .env.test DATABASE = myapp_test USERNAME = myuser PASSWORD = mypassword HOSTNAME = localhost POOL_SIZE = 10 POOL = Ecto.Adapters.SQL.Sandbox The above setup would likely commit the .env.test file so it was sure to override, and add .env to .gitignore , but other strategies are possible. The above example demonstrates developer settings appropriate for local development in the sample .env file, but a production deployment would only differ in its values : the shape of the file would be the same. The .env.test file is loaded when running tests, so its values override any of the values set in the .env . By using Dotenvy.env!/2 , a strong contract is created with the environment: the system running this app must have the designated environment variables set somehow, otherwise this app will not start (and a specific error will be raised). Using the nil-able variants of the type-casting (those ending with ? ) is an easy way to fall back to nil when the variable contains an empty string: env!(&quot;POOL&quot;, :module?) requires that the POOL variable is set, but it will return a nil if the value is an empty string. See Dotenvy.Transformer for more details.","ref":"strategies.html#dotenvs-for-all-environments","title":"Strategies - Dotenvs for All Environments","type":"extras"},{"doc":"One of the hurdles when dealing with Elixir releases is that only certain files are packaged into them. One solution to this is to specify additional directories to include in the release via the overlays option in your mix.exs , e.g. an envs/ directory which contains your dotenv files: ` ` ` elixir # mix.exs defp releases do [ myapp : [ overlays : [ &quot;envs/&quot; ] ] ] end ` ` ` Since these files are copied to the root of your release, the relative paths used in your runtime.exs will not be able to find them when your app is running in the context of a release. One solution to this is to rely on the RELEASE_ROOT system environment variable which is set when a release is run. If this value exists, it will represent the fully qualified path to your release; this variable will not be set when running your app locally (e.g. during development). We can use the presence of the RELEASE_ROOT to determine a directory prefix for where to look for our dotenv files, e.g.: ` ` ` elixir import Config import Dotenvy # For local development, read dotenv files inside the envs/ dir; # for releases, read them at the RELEASE_ROOT config_dir_prefix = System . fetch_env ( &quot;RELEASE_ROOT&quot; ) |&gt; case do :error -&gt; &quot;envs/&quot; { :ok , value } -&gt; IO . puts ( &quot;Loading dotenv files from \#{ value } &quot; ) &quot; \#{ value } /&quot; end source! ( [ &quot; \#{ config_dir_prefix } .env&quot; , &quot; \#{ config_dir_prefix } . \#{ config_env ( ) } .env&quot; , &quot; \#{ config_dir_prefix } . \#{ config_env ( ) } .local.env&quot; System . get_env ( ) ] ) ` ` ` Or more succinctly: ` ` ` elixir config_dir_prefix = System . get_env ( &quot;RELEASE_ROOT&quot; ) || &quot;envs/&quot; ` ` ` It can be safer to reference an absolute path, e.g. ` ` ` elixir config_dir_prefix = System . get_env ( &quot;RELEASE_ROOT&quot; ) || Path . expand ( &quot;./envs/&quot; ) &lt;&gt; &quot;/&quot; ` ` ` This is especially important when working with umbrella apps (see below).","ref":"strategies.html#releases","title":"Strategies - Releases","type":"extras"},{"doc":"Elixir Umbrella Projects consume configuration slightly differently due to how they are organized. In particular, you have to be very careful about relative paths when working in an umbrella project. Depending on what you're doing, the path may be relative to a single application instead of relative to the root of the repository. Using Path.expand/1 is a good way to anchor your config/runtime.exs to point to the root of the repository instead of it resolving to the root of a specific application within the umbrella. E.g. ` ` ` elixir env_dir_prefix = System . get_env ( &quot;RELEASE_ROOT&quot; ) || Path . expand ( &quot;./envs/&quot; ) &lt;&gt; &quot;/&quot; source! ( [ &quot; \#{ env_dir_prefix } \#{ config_env ( ) } .env&quot; , &quot; \#{ env_dir_prefix } \#{ config_env ( ) } .local.env&quot; , System . get_env ( ) ] ) ` ` `","ref":"strategies.html#umbrella-apps","title":"Strategies - Umbrella Apps","type":"extras"},{"doc":".env files (a.k.a. &quot;dotenv&quot;) store key-value pairs in a format descended from simple bash files that exported environment variables. This implementation cleaves closely to the format described by the original dotenv package, but it is not a direct match (by design). Typically, a dotenv ( .env ) file is formatted into simple key-value pairs: S3_BUCKET = YOURS3BUCKET SECRET_KEY = YOURSECRETKEYGOESHERE Optionally, you may add export in front of each line so you can source the file in Bash: export S3_BUCKET = YOURS3BUCKET export SECRET_KEY = YOURSECRETKEYGOESHERE","ref":"dotenv-file-format.html","title":"Dotenv File Format","type":"extras"},{"doc":"For the sake of portability (and sanity), environment variable names must consist solely of letters, digits, and the underscore ( _ ) and must not begin with a digit. In regex-speak, the names must match the following pattern: [ a - zA - Z_ ] + [ a - zA - Z0 - 9 _ ] * Example variable names DATABASE_URL # ok foobar # ok NO - WORK # &lt;-- invalid !!! Ü BER # &lt;-- invalid !!! 2 MUCH # &lt;-- invalid !!!","ref":"dotenv-file-format.html#variable-names","title":"Dotenv File Format - Variable Names","type":"extras"},{"doc":"Values are to the right of the equals sign. They may be quoted. Using single quotes will prevent variables from being interpolated. SIMPLE = xyz123 INTERPOLATED = &quot;Multiple \\n Lines and variable substitution: ${SIMPLE}&quot; NON_INTERPOLATED = &#39;raw text without variable interpolation&#39; MULTILINE = &quot;&quot;&quot; long text here, e.g. a private SSH key &quot;&quot;&quot;","ref":"dotenv-file-format.html#values","title":"Dotenv File Format - Values","type":"extras"},{"doc":"The following character strings will be interpreted (i.e. escaped) as specific codepoints in the same way you would expect if the values were assigned inside a script. Remember: when a text file is read, it is read as a series of utf8 encoded code points. Character sequences like \\n have no special meaning until they are &quot;escaped&quot; and a combination of codepoints is replaced by a single codepoint. \\n Linefeed (aka newline); &lt;&lt;92, 110&gt;&gt; -&gt; &lt;&lt;10&gt;&gt; \\r Carriage return; &lt;&lt;92, 114&gt;&gt; -&gt; &lt;&lt;13&gt;&gt; \\t Tab; -&gt; &lt;&lt;92, 116&gt;&gt; -&gt; &lt;&lt;9&gt;&gt; \\f Form feed; -&gt; &lt;&lt;92, 102&gt;&gt; -&gt; &lt;&lt;12&gt;&gt; \\b Backspace; -&gt; &lt;&lt;92, 98&gt;&gt; -&gt; &lt;&lt;8&gt;&gt; \\&quot; Double-quote; -&gt; &lt;&lt;92, 34&gt;&gt; -&gt; &lt;&lt;34&gt;&gt; \\' Single-quote; -&gt; &lt;&lt;92, 39&gt;&gt; -&gt; &lt;&lt;39&gt;&gt; \\\\ Backslash; -&gt; &lt;&lt;92, 92&gt;&gt; -&gt; &lt;&lt;92&gt;&gt; \\uFFFF Unicode escape (4 hex characters to denote the codepoint) If a backslash precedes any other character, that character will be interpreted literally: i.e. the backslash will be ignored and removed from output. Interpolation (a.k.a. Variable Substitution) Values left unquoted or wrapped in double-quotes will interpolate variables in the ${VAR} syntax. This can be useful for referencing existing system environment variables or to reference variables previously parsed. For example: USER = admin EMAIL = $ { USER } @example . org DATABASE_URL = &quot;postgres://${USER}@localhost/my_database&quot; CACHE_DIR = $ { PWD } / cache Multi-line values (e.g. private keys) can use the triple-quoted heredoc syntax: PRIVATE_KEY = &quot;&quot;&quot; -----BEGIN RSA PRIVATE KEY----- ... HkVN9... ... -----END DSA PRIVATE KEY----- &quot;&quot;&quot; Non-Interpolated If your values must retain ${} in their output, wrap the value in single quotes, e.g.: PASSWORD = &#39;!@G0${k}k&#39; MESSAGE_TEMPLATE = &#39;&#39;&#39; Hello ${PERSON}, Nice to meet you! &#39;&#39;&#39;","ref":"dotenv-file-format.html#escape-sequences","title":"Dotenv File Format - Escape Sequences","type":"extras"},{"doc":"The hash-tag # symbol denotes a comment when on its own line or when it follows a quoted value. It is not treated as a comment when it appears within quotes. # This is a comment SECRET_KEY = YOURSECRETKEYGOESHERE # also a comment SECRET_HASH = &quot;something-with-a-hash-#-this-is-not-a-comment&quot;","ref":"dotenv-file-format.html#comments","title":"Dotenv File Format - Comments","type":"extras"},{"doc":"The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Enables exception rescuing to report on problems with custom callback functions Moves Dotenvy.Transformer.Error to Dotenvy.Error to offer a simpler interface for devs who want to raise errors from custom transformer functions Improved documentation and examples for usage in umbrella apps Updates all internal options in the parser to use %Opts{} struct Improves test coverage Bumps Elixir version specified for local development in .tool-versions Updates dependencies to latest","ref":"changelog.html#v0-8-0","title":"Changelog - v0.8.0","type":"extras"},{"doc":"Formally defines a type for all supported conversions to improve documentation and specs Updates dependencies to latest Specifies Elixir 1.13 as required (simply because I can't get anything older to compile)","ref":"changelog.html#v0-7-0","title":"Changelog - v0.7.0","type":"extras"},{"doc":"Does away with the confusing :overwrite? and vars options in favor of a simple declarative/explicit inputs. source/2 now accepts ad-hoc maps as inputs. Updates dependencies including :ex_doc to take advantage of admonishment blocks. Various documentation cleanups/clarifications.","ref":"changelog.html#v0-6-0","title":"Changelog - v0.6.0","type":"extras"},{"doc":"Shifts storage of system environment variables to the application process dictionary and alters the reading of this data to help improve the security posture and avoid leaking env values. :side_effect option for source/2 and source!/2 function changed.","ref":"changelog.html#v0-5-0","title":"Changelog - v0.5.0","type":"extras"},{"doc":"Makes error messages more informative when unable to convert strings to integers or floats","ref":"changelog.html#v0-4-1","title":"Changelog - v0.4.1","type":"extras"},{"doc":"Adds support for custom transformer types by allowing an arity 1 function as the second argument to Dotenvy.Transformer.to/2. See Issue 2","ref":"changelog.html#v0-4-0","title":"Changelog - v0.4.0","type":"extras"},{"doc":"Renames Dotenvy.Transformer.to/2 to Dotenvy.Transformer.to!/2 to better communicate that it may raise an error. Returns key name in errors for easier troubleshooting. Tracks an error if the :require_files option lists a file not included in the files input (for sanity). Introduces Dotenvy.env!/3 (which is the same as Dotenvy.env/3 but with no defaults provided). This better communicates that it may raise an error (because internally it relies on Dotenvy.Transformer.to!/2 ) Deprecates Dotenvy.env/3 in favor of Dotenvy.env!/3","ref":"changelog.html#v0-3-0","title":"Changelog - v0.3.0","type":"extras"},{"doc":"Adds support for default type of :string to the Dotenvy.env!/2 and Dotenvy.env/3 functions.","ref":"changelog.html#v0-2-0","title":"Changelog - v0.2.0","type":"extras"},{"doc":"Initial release.","ref":"changelog.html#v0-1-0","title":"Changelog - v0.1.0","type":"extras"}]